@@ -6,15 +6,22 @@ incorporates RIF Positive Conditions defined in Section Positive Conditions
 """
 from PositiveConditions import *
 from rdflib import Variable, BNode, URIRef, Literal, Namespace,RDF,RDFS
-from FuXi.Rete.RuleStore import N3Builtin
+from rdflib.Graph import Graph
 def ExtractVariables(clause):
     pass
 
-class Ruleset:
+def HornFromN3(n3Stream):
+    from FuXi.Rete.RuleStore import SetupRuleStore
+    store,graph=SetupRuleStore(n3Stream)
+    store._finalize()
+    return Ruleset(n3Rules=store.rules,nsMapping=store.nsMgr)
+
+class Ruleset(object):
     """
     Ruleset ::= RULE*
     """
     def __init__(self,formulae=None,n3Rules=None,nsMapping=None):
+        from FuXi.Rete.RuleStore import N3Builtin
         self.nsMapping = nsMapping and nsMapping or {}        
         self.formulae = formulae and formulae or []
         if n3Rules:
@@ -39,7 +46,7 @@ class Ruleset:
         for f in self.formulae:
             yield f
 
-class Rule:
+class Rule(object):
     """
     RULE ::= 'Forall' Var* CLAUSE
     
@@ -72,10 +79,32 @@ class Rule:
                                     self.formula.head.n3())
 #        "Forall %s ( %r )"%(' '.join([var.n3() for var in self.declare]),
 #                               self.formula)
+
+    def __eq__(self,other):
+        return hash(self.formula)==hash(other.formula)
+        
+    def __hash__(self):
+        """
+        >>> a=Clause(And([Uniterm(RDFS.subClassOf,[Variable('C'),Variable('SC')]),
+        ...             Uniterm(RDF.type,[Variable('M'),Variable('C')])]),
+        ...        Uniterm(RDF.type,[Variable('M'),Variable('SC')]))
+        >>> b=Clause(And([Uniterm(RDFS.subClassOf,[Variable('C'),Variable('SC')]),
+        ...             Uniterm(RDF.type,[Variable('M'),Variable('C')])]),
+        ...        Uniterm(RDF.type,[Variable('M'),Variable('SC')]))
+        >>> d=set()
+        >>> d.add(a)
+        >>> b in d
+        True
+        >>> hash(a) == hash(b)
+        True
+        
+        """
+        return hash(self.formula)
+
     def __repr__(self):
         return "Forall %s ( %r )"%(' '.join([var.n3() for var in self.declare]),
                                self.formula)    
-class Clause:
+class Clause(object):
     """
     Facts are *not* modelled formally as rules with empty bodies
     
@@ -93,13 +122,11 @@ class Clause:
     def __init__(self,body,head):
         self.body = body
         self.head = head
-        #assert isinstance(head,Atomic),repr(head)
-        assert isinstance(body,Condition)
         from FuXi.Rete.Network import HashablePatternList
-        antHash=HashablePatternList([term.toRDFTuple() 
-                            for term in body])
-        consHash=HashablePatternList([term.toRDFTuple() 
-                            for term in head])                                                                                            
+        antHash=HashablePatternList(
+                    [term.toRDFTuple() for term in body],skipBNodes=True)
+        consHash=HashablePatternList(
+                    [term.toRDFTuple() for term in head],skipBNodes=True)                                                                                            
         self._hash = hash(antHash) ^ hash(consHash)
         
     def __eq__(self,other):
@@ -120,6 +147,9 @@ class Clause:
         >>> hash(a) == hash(b)
         True
         
+        >>> d={a:True}
+        >>> b in d
+        True
         """
         return self._hash
         
@@ -127,6 +157,8 @@ class Clause:
         return (self.body,self.head)
         
     def __repr__(self):
+        if isinstance(self.body,SetOperator) and not len(self.body):
+            return "%r :-"%self.head
         return "%r :- %r"%(self.head,self.body)
     
     def n3(self):
