@@ -152,7 +152,8 @@ nsBinds = {
     'skos': 'http://www.w3.org/2004/02/skos/core#',
     'rdf' : RDF.RDFNS,
     'rdfs': RDFS.RDFSNS,
-    'owl' : OWL_NS,       
+    'owl' : OWL_NS,
+    'list' : URIRef('http://www.w3.org/2000/10/swap/list#'),       
     'dc'  : "http://purl.org/dc/elements/1.1/",
 }
 
@@ -405,6 +406,19 @@ def AllProperties(graph):
                            graph=graph,
                            baseType=bType)            
     
+class ClassNamespaceFactory(Namespace):
+    def term(self, name):
+        return Class(URIRef(self + name))
+
+    def __getitem__(self, key, default=None):
+        return self.term(key)
+
+    def __getattr__(self, name):
+        if name.startswith("__"): # ignore any special Python names!
+            raise AttributeError
+        else:
+            return self.term(name)    
+    
 def CastClass(c,graph=None):
     graph = graph is None and c.factoryGraph or graph
     for kind in graph.objects(subject=classOrIdentifier(c),
@@ -467,6 +481,31 @@ class Class(AnnotatibleTerms):
         if complementOf:
             self.complementOf    = complementOf
         self.comment = comment and comment or []
+        
+    def _get_extent(self):
+        for member in self.graph.subjects(predicate=RDF.type,
+                                          object=self.identifier):
+            yield member
+    def _set_extent(self,other):
+        if not other:
+            return
+        for m in other:
+            self.graph.add((m,RDF.type,self.identifier))
+            
+    extent = property(_get_extent, _set_extent)            
+    
+    def __hash__(self):
+        """
+        >>> b=Class(OWL_NS.Restriction)
+        >>> c=Class(OWL_NS.Restriction)
+        >>> len(set([b,c]))
+        1
+        """
+        return hash(self.identifier)
+
+    def __eq__(self, other):
+        assert isinstance(other,Class)
+        return self.identifier == other.identifier
     
     def __iadd__(self, other):
         assert isinstance(other,Class)
@@ -633,10 +672,10 @@ class Class(AnnotatibleTerms):
         return (isinstance(self.identifier,BNode) and "Some Class " or "Class: %s "%self.qname)+klassDescr
 
 class OWLRDFListProxy(object):
-    def __init__(self,rdfList,members=None,graph=Graph()):
+    def __init__(self,rdfList,members=None):
         members = members and members or []
         if rdfList:
-            self._rdfList = Collection(graph,rdfList[0])
+            self._rdfList = Collection(self.graph,rdfList[0])
             for member in members:
                 if member not in self._rdfList:
                     self._rdfList.append(classOrIdentifier(member))
@@ -692,7 +731,7 @@ class EnumeratedClass(Class,OWLRDFListProxy):
         Class.__init__(self,identifier,graph = graph)
         members = members and members or []
         rdfList = list(self.graph.objects(predicate=OWL_NS.oneOf,subject=self.identifier))
-        OWLRDFListProxy.__init__(self, rdfList, members, graph = graph)
+        OWLRDFListProxy.__init__(self, rdfList, members)
     def __repr__(self):
         """
         Returns the Manchester Syntax equivalent for this class
@@ -724,7 +763,7 @@ class BooleanClass(Class,OWLRDFListProxy):
         self._operator = operator
         rdfList = list(self.graph.objects(predicate=operator,subject=self.identifier))
         assert not members or not rdfList,"This is a previous boolean class description!"+repr(Collection(self.graph,rdfList[0]).n3())        
-        OWLRDFListProxy.__init__(self, rdfList, members, graph = graph)
+        OWLRDFListProxy.__init__(self, rdfList, members)
 
     def isPrimitive(self):
         return False
@@ -812,6 +851,7 @@ class Restriction(Class):
                       (minCardinality,OWL_NS.minCardinality)]
         validRestrProps = [(i,oTerm) for (i,oTerm) in restrTypes if i] 
         assert len(validRestrProps) < 2
+        self.restrictionRange = validRestrProps
         for val,oTerm in validRestrProps:
             self.graph.add((self.identifier,oTerm,classOrTerm(val)))   
         if (self.identifier,RDF.type,OWL_NS.Restriction) not in self.graph:
@@ -820,6 +860,18 @@ class Restriction(Class):
     def isPrimitive(self):
         return False
 
+    def __eq__(self, other):
+        """
+        Equivalence of restrictions is determined by equivalence of the property 
+        in question and the restriction 'range'
+        """
+        assert isinstance(other,Class)
+        if isinstance(other,Restriction):
+            return other.onProperty == self.onProperty and \
+                   other.restrictionRange == self.restrictionRange
+        else:
+            return False
+
     def _get_onProperty(self):
         return list(self.graph.objects(subject=self.identifier,predicate=OWL_NS.onProperty))[0]
     def _set_onProperty(self, prop):
@@ -977,7 +1029,12 @@ class Property(AnnotatibleTerms):
             rt.append('ObjectProperty( %s annotation(%s)'\
                        %(self.qname,first(self.comment) and first(self.comment) or ''))
             if first(self.inverseOf):
-                rt.append("  inverseOf( %s )%s"%(first(self.inverseOf),
+                twoLinkInverse=first(first(self.inverseOf).inverseOf)
+                if twoLinkInverse and twoLinkInverse.identifier == self.identifier:
+                    inverseRepr=first(self.inverseOf).qname
+                else:
+                    inverseRepr=repr(first(self.inverseOf))
+                rt.append("  inverseOf( %s )%s"%(inverseRepr,
                             OWL_NS.Symmetric in self.type and ' Symmetric' or ''))
             for s,p,roleType in self.graph.triples_choices((self.identifier,
                                                             RDF.type,
